from .board import Board, Piece
from .tictactoe import TicTacToe


class MisereTicTacToe(TicTacToe):
    '''Misere TicTacToe, where the player wins if the opponent 'wins' in normal rules. '''
    def __init__(self, m=3, n=3, k=3) -> None:
        super().__init__(m, n, k)
    
    def checkwin(self) -> bool:
        if super().checkwin():
            self.winner = Piece.previous(self.turn)


class UltimateTicTacToe(TicTacToe):
    ''' Ultimate TicTacToe. '''
    def __init__(self, m=3, n=3, k=3, where=[0, 0]) -> None:
        ''' Initialize Ultimate TicTacToe.
        self.localboards contain regular local boards; turn parameter for them is set to default and unused.
        self.board contain information about who won for each local board.
        self.where attribute determine on which local board the next move is pushed.
        '''
        super().__init__(m, n, k)
        self.localboards = [[TicTacToe(m, n, k) for _ in range(m)] for _ in range(n)]
        self.where = where

    def movevalid(self, x, y) -> bool:
        ''' Return True if move is valid. Note that we can move on already winned small boards. '''
        return self.localboards[self.where[0]][self.where[1]].movevalid(x, y)

    def move(self, x, y) -> bool:
        '''Make move, update attributes, choose small board for next move. Return True if move is successful. '''
        if self.movevalid(x, y):
            smallboard = self.localboards[self.where[0]][self.where[1]]
            smallboard[x, y] = self.turn
            self.where = [x, y]
            smallboard.checkwin()
            smallboard.checkdraw()

            self.board[x][y] = smallboard.winner
            self.checkwin()
            self.checkdraw()
            self.changeturn()
            self.move_history.append((x,y))
            self.move_count += 1
            return True
        return False

    def __str__(self) -> str:
        printed = []
        for (i, j) in self.moves_coords():
                printed.append(f'------------- LOCAL BOARD {i} {j} -------------\n')
                printed.append('Local attributes:')
                printed.append(str(self.localboards[i][j]))
        printed.append('\n=========== Global attributes: ===========')
        printed.append(super().__str__())
        return ''.join(printed)